@@ -7,8 +7,28 @@ import requests
 import os
 from abstract_gui import AbstractWindowManager,sg,ensure_nested_list,expandable,make_component,text_to_key
 from .abstract_blockchain_functions import *
+class APIURLManager:
+    def __init__(self,api_mgr):
+        self.api_mgr = api_mgr or APIBridge()
+        self.url_manager={}
+    def add_url(self,url,output):
+        self.scanner = self.api_mgr.rpc_mgr.scanner
+        if self.scanner not in self.url_manager:
+            self.url_manager[self.scanner]=[]
+        url_parts = url.split('?')
+        scanner=strip_web(url_parts[0])
+        self.url_manager[self.scanner].append({"action":"","url":[],"output":[],"inputs":{}})
+        self.url_manager[self.scanner][-1]["url"].append(url)
+        self.url_manager[self.scanner][-1]["output"].append(output)
+        for item in url_parts[-1].split('&'):
+            item_parts = item.split('=')
+            if item_parts[0] == 'action':
+                self.url_manager[self.scanner][-1]['action']=item_parts[-1]
+            else:
+                self.url_manager[self.scanner][-1]["inputs"][item_parts[0]]=item_parts[-1]
+            
 class ApiGUI:
-    def __init__(self,api_mgr=None,rpc_mgr=None,
+    def __init__(self,api_url_mgr=None,api_gui=False,api_mgr=None,rpc_mgr=None,
                  rpc_js:dict=None,
                  rpc_gui:bool=False,
                  address:str=None,
@@ -17,6 +37,7 @@ class ApiGUI:
                  end_block:str=None,
                  api_key:str=None,
                  env_key:str=None):
+        self.api_url_mgr = api_url_mgr or APIURLManager(api_mgr=api_mgr)
         self.api_mgr = api_mgr
         if self.api_mgr == None:
             self.api_mgr=APIBridge(rpc_mgr=rpc_mgr,
@@ -28,6 +49,8 @@ class ApiGUI:
                  end_block=end_block,
                  api_key=api_key,
                  env_key=env_key)
+        self.contract_address = contract_address
+        self.address=address
         self.rpc_mgr_gui=RPCGUIManager(rpc_mgr=self.api_mgr.rpc_mgr,gui_window=False)
         self.rpc_gui_keys=[]
         self.section='APIMGR'
@@ -41,11 +64,16 @@ class ApiGUI:
             [[make_component("Frame","api output",layout=output,**expandable(size=(None,180)))],sg.Text('Select API Call Type:'), sg.Combo(values=api_keys,default_value=api_keys[0], key='API_SELECT', enable_events=True)],
             [sg.Frame('Parameters:', [[sg.Column(self.generate_api_gui(apiCallDesc[api_keys[0]]), key='API_GUI')]])]
         ]
-        
         self.windows_mgr=AbstractWindowManager()
         self.window_name = self.windows_mgr.add_window(title='API Call GUI', layout=self.layout,close_events=['Exit',"OK"],event_handlers=[self.while_window])
+
+        if api_gui:
+            self.pop_up_gui()
+        
+
+    def pop_up_gui(self):
         self.windows_mgr.while_window(window_name=self.window_name)
-        self.api_url = self.windows_mgr.search_closed_windows(window_name=self.window_name)['values']['API_URL']
+        return self.windows_mgr.search_closed_windows(window_name=self.window_name)['values']['API_URL']
     def generate_api_gui(self,api_desc):
         """
         Function to generate GUI layout based on the selected API description.
@@ -156,9 +184,9 @@ class ApiGUI:
             if key:
                 text+=self.values[key]
         if self.values['-SCANNER_TOGGLE-']:
-            scanner = self.api_mgr.rpc_mgr.scanner
-            if scanner:
-                scanner_protocol = f"https://{('api.' if 'api' != scanner else '')}{scanner}/api?"
+            self.scanner = self.api_mgr.rpc_mgr.scanner
+            if self.scanner:
+                scanner_protocol = f"https://{('api.' if 'api' != self.scanner else '')}{self.scanner}/api?"
                 text=scanner_protocol+text
         # Here's where you would combine the base URL with the user's inputs 
         # to generate the full API URL. I'm just simulating this step.
@@ -167,9 +195,9 @@ class ApiGUI:
         output=None
         generated_url= self.values['API_URL']
         value  = self.values['-SCANNER_TOGGLE-']
-        scanner,scanner_protocol = self.api_mgr.rpc_mgr.scanner,None
-        if scanner:
-            scanner_protocol = f"https://{('api.' if 'api' != scanner else '')}{scanner}/api?"
+        self.scanner,scanner_protocol = self.api_mgr.rpc_mgr.scanner,None
+        if self.scanner:
+            scanner_protocol = f"https://{('api.' if 'api' != self.scanner else '')}{self.scanner}/api?"
         if value:
             if scanner_protocol:
                 if '/api?' in generated_url:
@@ -203,7 +231,9 @@ class ApiGUI:
             self.values['-SCANNER_TOGGLE-'],self.values["-API_KEY_TOGGLE-"]=True,True
             generated_url = self.get_generated_url_output()
             self.values['-SCANNER_TOGGLE-'],self.values["-API_KEY_TOGGLE-"]=scanner_toggle,api_key_toggle
-            window['-API_OUTPUT-'].update(self.api_mgr.make_request(generated_url))
+            output = make_request(generated_url)
+            self.api_url_mgr.add_url(url=generated_url,output=output)
+            window['-API_OUTPUT-'].update(output)
         elif event == 'API_SELECT':
             selected_api = values['API_SELECT']
             window['API_GUI'].update(self.generate_api_gui(apiCallDesc[selected_api]))
@@ -257,6 +287,7 @@ class ApiGUI:
            self.api_mgr.get_api_key()
 class APIBridge:
     def __init__(self,rpc_mgr:str=None,
+                 api_gui_mgr=None,
                  api_data_type:str=None,
                  api_key:str=None,
                  env_key:str=None,
@@ -274,7 +305,8 @@ class APIBridge:
         
         self.get_api_key_network=[]
         self.rate_limiter = DynamicRateLimiterManager()
-        
+        self.rpc_mgr = rpc_mgr or RPCBridge(rpc_js=rpc_js)
+        self.api_gui_mgr = api_gui_mgr or ApiGUI(api_mgr=self)
         
         self.rpc_mgr=rpc_mgr
         self.rpc_js=rpc_js
@@ -289,6 +321,7 @@ class APIBridge:
         
         self.contract_address=contract_address
         self.address=address
+        
         self.update_address(address=address,contract_address=contract_address)
         
         self.start_block=start_block
@@ -322,7 +355,7 @@ class APIBridge:
         self.api_service_tracker[self.service_name] = self.api_key
         # Configure rate limiter for the service
         self.rate_limiter.add_service(service_name=self.service_name, low_limit=low_limit, high_limit=high_limit)
-
+        return self.service_name
     def update_address(self,address=None,contract_address=None):
         if address == False:
             self.address=None
@@ -336,6 +369,7 @@ class APIBridge:
         else:
             if self.contract_address != contract_address and contract_address != None:
                 self.contract_address = checksum(self.contract_address)
+        return self.contract_address
     def update_api_call_partameters(self,rpc_mgr=None,rpc_js=None,address=[],contract_address=None,start_block=None,end_block=None,api_data_type=None,api_gui=None,**kwargs):
         if rpc_mgr or rpc_js:
             self.update_rpc(rpc_mgr=rpc_mgr,rpc_js=rpc_js)
@@ -359,7 +393,7 @@ class APIBridge:
     def make_api_call(self,api_url=None,api_gui=False,api_url_data=None):
         self.api_url_data = api_url_data or self.api_url_data
         if api_gui:
-            self.api_url=ApiGUI(api_mgr=self)
+            self.api_url=self.api_gui_mgr.pop_up_gui()
         try:
             if self.api_url == None:
                 self.api_url = f"https://{('api.' if 'api' != self.rpc_mgr.scanner[:len('api')] else '')}{self.rpc_mgr.scanner}/api?{self.api_url_data}&apikey={self.api_key}"
@@ -375,4 +409,4 @@ class APIBridge:
             except Exception as e:
                 logging.error(f"Error in making request: {e}")
                 return None
-            
+
